﻿unit OblongTemperature;

interface

uses
  SysUtils, Classes;

const
  AbsoluteZeroC: Double = -273.15; //°C
  AbsoluteZeroK: Double = 0; //°C

type
  TTemperature = class;

  /// A procedural type that underlies the TTemperature.OnInvalidTemp event.
  TInvalidTempEvent = procedure(Sender: TTemperature; Temp: Double;
    var Handled: Boolean) of object;

  /// A class for converting temperatures from one scale to another.
  /// The TTemperature class is a non-visual component.
  TTemperature = class(TComponent)
  strict private
    Fkelvin: Double;
    FOnInvalidTemp: TInvalidTempEvent;
  strict protected
    /// Routine to signal OnInvalidTemp event
    procedure DoInvalidTemp(InvalidTemp: Double); virtual;
    /// Property reader for Celsius and Centigrade properties
    function GetCelsius: Double;
    /// Property reader for Fahrenheit property
    function GetFahrenheit: Double;
    /// Property reader for kelvin property
    function Getkelvin: Double;
    /// Property reader for Newton property
    function GetNewton: Double;
    /// Property reader for Romer property
    function GetRomer: Double;
    /// Property reader for Reaumur property
    function GetReaumur: Double;
    /// Property reader for Rankine property
    function GetRankine: Double;
    /// Property writer for Celsius and Centigrade properties
    procedure SetCelsius(const Value: Double);
    /// Property writer for Fahrenheit property
    procedure SetFahrenheit(const Value: Double);
    /// Property writer for kelvin property
    procedure Setkelvin(const Value: Double);
    /// Property writer for Newton property
    procedure SetNewton(const Value: Double);
    /// Property writer for Romer property
    procedure SetRomer(const Value: Double);
    /// Property writer for Reaumur property
    procedure SetReaumur(const Value: Double);
    /// Property writer for Rankine property
    procedure SetRankine(const Value: Double);
    /// Property reader for Delisle property
    function GetDelisle: Double;
    /// Property writer for Delisle property
    procedure SetDelisle(const Value: Double);
  public
    /// This is a custom constructor that initialises the internal field
    constructor Create(AOwner: TComponent); override;
  published
    /// Temperature in degrees Celsius (or Centigrade).
    /// Notes: Proposed (in a slightly different form) in 1742 by Swedish astronomer Anders Celsius.
    /// In 1948 officially renamed from Centigrade.
    /// The freezing point of water is 0 °C, the boiling point is 100 °C
    /// and absolute zero is at -273.15 °C.
    /// One degree Celsius equals one kelvin.
    /// Standard abbreviation is °C
    property Celsius: Double read GetCelsius write SetCelsius stored False;
    /// Temperature in degrees Centigrade (or Celsius).
    /// Notes: Proposed (in a slightly different form) in 1742 by Swedish astronomer Anders Celsius.
    /// In 1948 Centigrade was officially renamed to Celsius.
    /// The freezing point of water is 0 °C, the boiling point is 100 °C
    /// and absolute zero is at -273.15 °C.
    /// One degree Centigrade equals one kelvin.
    /// Standard abbreviation is °C
    property Centigrade: Double read GetCelsius write SetCelsius stored False;
    /// Temperature in degrees Delisle.
    /// Notes: Invented in 1732 by French astronomer Joseph-Nicolas Delisle.
    /// It is similar to that of Réaumur.
    /// The freezing point of water is 150 °De, and the boiling point is 0 °De
    /// and absolute zero is at 559.725 °De.
    /// One degree Delisle equals −2/3 of a kelvin or degree Celsius.
    /// Standard abbreviation is °De
    property Delisle: Double read GetDelisle write SetDelisle stored False;
    /// Temperature in degrees Fahrenheit.
    /// Notes: Proposed in 1708 by German physicist Gabriel Fahrenheit.
    /// The freezing point of water is 32 °F, the boiling point is 212 °F
    /// and absolute zero is -459.67 °F.
    /// One degree Fahrenheit equals 5/9th of a kelvin or degree Celsius.
    /// Standard abbreviation is °F
    property Fahrenheit: Double read GetFahrenheit write SetFahrenheit stored False;
    /// Temperature in kelvin units.
    /// Notes: Introduced in 1954 as the SI unit of temperature.
    /// Named after William Thomson, 1st Baron Kelvin.
    /// The freezing point of water is 273.15 K, the boiling point is 373.15 K
    /// and absolute zero (when molecular motion stops) is 0 K.
    /// One kelvin is the fraction 1/273.16 of the thermodynamic temperature of the triple point of water.
    /// One kelvin equals one degree Celsius.
    /// Standard abbreviation is K
    property kelvin: Double read Getkelvin write Setkelvin;
    /// Temperature in degrees Newton.
    /// Notes: Proposed around 1700 by Isaac Newton.
    /// The freezing point of water is 0 °N, the boiling point is 33 °N
    /// and absolute zero is -90.1395 °N.
    /// 1 degree Newton equals 100/33rd of a kelvin or a degree Celcius.
    /// Standard abbreviation is °N
    property Newton: Double read GetNewton write SetNewton stored False;
    /// Temperature in degrees Rankine.
    /// Notes: Proposed in 1859 by Scottish engineer and physicist William John Macquorn Rankine.
    /// The freezing point of water is 491.67 °R, the boiling point is 671.67 °R
    /// and absolute zero is 0 °R.
    /// One degree Rankine equals one degree Fahrenheit.
    /// Standard abbreviation is °R
    property Rankine: Double read GetRankine write SetRankine stored False;
    /// Temperature in degrees Réaumur.
    /// Notes: Proposed in 1731 by René Antoine Ferchault de Réaumur.
    /// Also known as the "division octogesimale" or "octogesimal division".
    /// The freezing point of water is 0 °Ré, the boiling point 80 °Ré
    /// and absolute zero is -218.52 °Ré.
    /// One degree Reaumur is 1.25 degrees Celsius or kelvins.
    /// Standard abbreviation is °R, but °Ré allows unique identification
    property Reaumur: Double read GetReaumur write SetReaumur stored False;
    /// Temperature in degrees Rømer.
    /// Notes: Proposed in 1701 by Danish astronomer Ole Christensen Rømer.
    /// The freezing point of water is 7.5 °Rø, the boiling point 60 °Rø
    /// and absolute zero is -135.9038 °Rø.
    /// One degree Rømer equals 40/21st of a kelvin or degree Celsius.
    /// Standard abbreviation is °R, but °Rø allows unique identification
    property Romer: Double read GetRomer write SetRomer stored False;
    /// OnInvalidTemp is fired when a temperature below absolute
    /// zero is assigned to any of the temperature properties
    property OnInvalidTemp: TInvalidTempEvent read FOnInvalidTemp write FOnInvalidTemp;
  end;

implementation

uses
  Math, ConvUtils, StdConvs;

{ TTemperature }

resourcestring
  STempTooLow = 'Temperature less than absolute zero is invalid';

const
  RoundToDigits = -4;

constructor TTemperature.Create(AOwner: TComponent);
begin
  inherited;
  FKelvin := 273.15
end;

function TTemperature.GetCelsius: Double;
begin
  Result := RoundTo(Convert(Kelvin, tuKelvin, tuCelsius), RoundToDigits)
end;

procedure TTemperature.SetCelsius(const Value: Double);
begin
  Kelvin := Convert(Value, tuCelsius, tuKelvin)
end;

function TTemperature.GetDelisle: Double;
begin
  Result := RoundTo((373.15 - Kelvin) * 3/2, RoundToDigits)
end;

procedure TTemperature.SetDelisle(const Value: Double);
begin
  Kelvin := 373.15 - (Value * 2/3)
end;

function TTemperature.GetFahrenheit: Double;
begin
  Result := RoundTo(Convert(FKelvin, tuKelvin, tuFahrenheit), RoundToDigits)
end;

procedure TTemperature.SetFahrenheit(const Value: Double);
begin
  Kelvin := Convert(Value, tuFahrenheit, tuKelvin)
end;

function TTemperature.Getkelvin: Double;
begin
  Result := RoundTo(Fkelvin, RoundToDigits)
end;

procedure TTemperature.Setkelvin(const Value: Double);
begin
  if Value < AbsoluteZeroK then
    DoInvalidTemp(Value)
  else
    FKelvin := Value;
end;

function TTemperature.GetNewton: Double;
begin
  Result := RoundTo((Kelvin - 273.15) * 33/100, RoundToDigits)
end;

procedure TTemperature.SetNewton(const Value: Double);
begin
  Kelvin := (Value * 100/33) + 273.15
end;

function TTemperature.GetRankine: Double;
begin
  Result := RoundTo(Convert(FKelvin, tuKelvin, tuRankine), RoundToDigits)
end;

procedure TTemperature.SetRankine(const Value: Double);
begin
  Kelvin := Convert(Value, tuRankine, tuKelvin)
end;

function TTemperature.GetReaumur: Double;
begin
  Result := RoundTo(Convert(FKelvin, tuKelvin, tuReaumur), RoundToDigits)
end;

procedure TTemperature.SetReaumur(const Value: Double);
begin
  Kelvin := Convert(Value, tuReaumur, tuKelvin)
end;

function TTemperature.GetRomer: Double;
begin
  Result := RoundTo(((Kelvin - 273.15) * 21/40) + 7.5, RoundToDigits)
end;

procedure TTemperature.SetRomer(const Value: Double);
begin
 Kelvin := ((Value - 7.5) * 40/21) + 273.15
end;

procedure TTemperature.DoInvalidTemp(InvalidTemp: Double);
var
  Handled: Boolean;
begin
  Handled := False;
  if Assigned(FOnInvalidTemp) then
    FOnInvalidTemp(Self, InvalidTemp, Handled);
  if not Handled then
    raise EInvalidArgument.Create(STempTooLow)
end;

end.
